//+------------------------------------------------------------------+
//|                                                    SingleBar.mq5 |
//|                        Copyright 2020, MetaQuotes Software Corp. |
//|                            https://www.mql5.com/ru/articles/7535 |
//+------------------------------------------------------------------+
#property copyright "Copyright 2020, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.0"


#include <MT4Bridge/MT4Orders.mqh>
#include <MT4Bridge/MT4Time.mqh>
#include <Marketeer/Converter.mqh>
#include <OLAP/SingleNextBarDemo.mqh>


input string BuyHours = "";
input string SellHours = "";
input double Lot = 0.1;
input uint ActiveDayOfWeek = 0; // ActiveDayOfWeek (1-mon, 2-tue... 5-fri)
input uint AllowedMaxSpread = 10;


datetime lastBar = 0;

int buyHours[], sellHours[];
int singleStrategy;


int MathSign(const double x)
{
  return x > 0 ? +1 : (x < 0 ? -1 : 0);
}

int OnInit()
{
  lastBar = 0;
  
  const int trend = parseHours(BuyHours, buyHours);
  const int reverse = parseHours(SellHours, sellHours);
  singleStrategy = MathSign(trend) - MathSign(reverse);
  
  return trend > 0 || reverse > 0 ? INIT_SUCCEEDED : INIT_PARAMETERS_INCORRECT;
}

double plus = 0;
double minus = 0;
int countadd = 0;
int countsub = 0;

void OnDeinit(const int reason)
{
  Converter<ulong,double> converter;
  const double infinity = converter[0x7FF0000000000000];
  if(!singleStrategy)
  {
    Print("Warning! Mixed strategy used, stats are merged (not informative)");
  }
  else
  if(StringLen(SellHours) > 0)
  {
    Print("Reversal strategy is enabled: PF and WIN% are inversed to actual values");
  }
  Print("PF: ", (minus > 0 ? plus / minus : (plus > 0 ? infinity : 0)), ", WIN%: ", (countadd + countsub > 0 ? (float)(100.0 * countadd / (countadd + countsub)) : 0));
}

bool isNewBar()
{
  if(lastBar != iTime(_Symbol, 0, 0))
  {
    lastBar = iTime(_Symbol, 0, 0);
    return true;
  }
  return false;
}


void OnTick()
{
  MqlTick tick;
  if(!SymbolInfoTick(_Symbol, tick)) return;
  
  if(AllowedMaxSpread > 0
  && (tick.ask - tick.bid) / _Point > AllowedMaxSpread) return; // wait for good conditions

  if(!isNewBar()) return;

  const int h = TimeHour(lastBar);

  int mode = 0;
  
  if(ArrayFind(buyHours, h) > -1)
  {
    mode = +1;
  }
  else
  if(ArrayFind(sellHours, h) > -1)
  {
    mode = -1;
  }

  if(ActiveDayOfWeek != 0 && ActiveDayOfWeek != _TimeDayOfWeek()) mode = 0; // skip all days except specified

  // pick up existing orders (if any)
  const int direction = CurrentOrderDirection();
  
  if(mode == 0)
  {
    if(direction != 0)
    {
      const double tradedBar = iOpen(_Symbol, _Period, 0) - iOpen(_Symbol, _Period, 1);
      const double spread = MathMax(iSpread(_Symbol, _Period, 1), iSpread(_Symbol, _Period, 0)) * _Point;
      const double value = tradedBar - singleStrategy * spread;
      
      string status;
      
      if(value >= 0)
      {
        plus += value;
        countadd++;

        if(OrderProfit() < 0)
        {
          status = "-buy";
        }
        else
        {
          status = "+buy";
        }
      }
      else
      {
        minus -= value;
        countsub++;

        if(OrderProfit() > 0)
        {
          status = "+sell";
        }
        else
        {
          status = "-sell";
        }
      }

      const int orderPoints = (OrderType() == OP_SELL ? -1 : +1) * (int)(((float)(OrderClosePrice() - OrderOpenPrice())) / (float)_Point);
      const int barPoints = (int)((float)tradedBar / (float)_Point);
      Print("Status: ", status, " ", (OrderType() == OP_BUY ? "BUY" : "SELL"), " ", (float)value, ShortToString(0xB1), (float)spread, " Order: ", orderPoints, " ", (float)OrderClosePrice(), " ", (float)OrderOpenPrice(),
            " Bar: ", barPoints, " ", (float)iOpen(_Symbol, _Period, 0), " ", (float)iOpen(_Symbol, _Period, 1));
      
      OrdersCloseAll();
    }
    return;
  }
  
  if(direction != 0) // there exist open orders
  {
    if(mode == direction) // keep direction
    {
      return; // existing trade goes on
    }
    OrdersCloseAll();
  }
  
  
  const int type = mode > 0 ? OP_BUY : OP_SELL;
  
  const double p = type == OP_BUY ? SymbolInfoDouble(_Symbol, SYMBOL_ASK) : SymbolInfoDouble(_Symbol, SYMBOL_BID);
    
  OrderSend(_Symbol, type, Lot, p, 100, 0, 0);
}
